@@ -1,7 +1,7 @@
 /*
  * valuepair.c	Functions to handle VALUE_PAIRs
  *
- * Version:	$Id: valuepair.c,v 1.133 2008/01/04 13:21:30 aland Exp $
+ * Version:	$Id: valuepair.c,v 1.136 2008/01/20 09:04:14 aland Exp $
  *
  *   This library is free software; you can redistribute it and/or
  *   modify it under the terms of the GNU Lesser General Public
@@ -21,7 +21,7 @@
  */
 
 #include <freeradius-devel/ident.h>
-RCSID("$Id: valuepair.c,v 1.133 2008/01/04 13:21:30 aland Exp $")
+RCSID("$Id: valuepair.c,v 1.136 2008/01/20 09:04:14 aland Exp $")
 
 #include	<freeradius-devel/libradius.h>
 
@@ -909,7 +909,7 @@ VALUE_PAIR *pairparsevalue(VALUE_PAIR *vp, const char *value)
 			break;
 
 		case PW_TYPE_BYTE:
-			if ((value[0] == '0') && (value[1] == 'x')) {
+			if (value && (value[0] == '0') && (value[1] == 'x')) {
 				goto do_octets;
 			}
 
@@ -1174,6 +1174,7 @@ static VALUE_PAIR *pairmake_any(const char *attribute, const char *value,
 	const char	*p = attribute;
 	char		*q;
 	VALUE_PAIR	*vp;
+	DICT_ATTR	*da;
 
 	/*
 	 *	Unknown attributes MUST be of type 'octets'
@@ -1183,6 +1184,12 @@ static VALUE_PAIR *pairmake_any(const char *attribute, const char *value,
 		return NULL;
 	}
 
+	da = dict_attrbyname(attribute);
+	if (da) {
+		attr = da->attr;
+		goto raw;
+	}
+
 	attr = vendor = 0;
 
 	/*
@@ -1284,6 +1291,7 @@ static VALUE_PAIR *pairmake_any(const char *attribute, const char *value,
 
 	attr |= vendor << 16;
 
+ raw:
 	/*
 	 *	We've now parsed the attribute properly, Let's create
 	 *	it.  This next stop also looks the attribute up in the
@@ -1396,6 +1404,11 @@ VALUE_PAIR *pairmake(const char *attribute, const char *value, int operator)
 		return pairmake_any(attribute, value, operator);
 	}
 
+	if (value && (value[0] == '0') && (value[1] == 'x') &&
+	    (da->type != PW_TYPE_OCTETS)) {
+		return pairmake_any(attribute, value, operator);
+	}
+
 	if ((vp = pairalloc(da)) == NULL) {
 		librad_log("out of memory");
 		return NULL;
@@ -1851,7 +1864,6 @@ int paircmp(VALUE_PAIR *one, VALUE_PAIR *two)
 	case PW_TYPE_OCTETS:
 	{
 		size_t length;
-		const uint8_t *p, *q;
 
 		if (one->length < two->length) {
 			length = one->length;
@@ -1859,11 +1871,10 @@ int paircmp(VALUE_PAIR *one, VALUE_PAIR *two)
 			length = two->length;
 		}
 
-		p = two->vp_octets;
-		q = one->vp_octets;
-		while (length) {
-			compare = ((int) *p) - ((int) *q);
-			if (compare != 0) goto type_switch;
+		if (length) {
+			compare = memcmp(two->vp_octets, one->vp_octets,
+					 length);
+			if (compare != 0) break;
 		}
 
 		/*
@@ -1913,7 +1924,6 @@ int paircmp(VALUE_PAIR *one, VALUE_PAIR *two)
 	/*
 	 *	Now do the operator comparison.
 	 */
- type_switch:
 	switch (one->operator) {
 	case T_OP_CMP_EQ:
 		return (compare == 0);
